package com.astrum.io;

public class IOUART extends IOStream {
	public enum Ports {
		UART1("/dev/ttyS3"), UART2("/dev/ttyS4");
		private final String text;

		private Ports(final String text) {
			this.text = text;
		}

		@Override
		public String toString() {
			return text;
		}

		public static Ports index(String port) {
			if (port != null) {
				for (Ports i : Ports.values()) {
					if (i.name().equals(port)) {
						return i;
					}
				}
			}
			return null;
		}
	}

	public enum Parity {
		Even, Mark, None, Odd, Parity, Space
	}

	public enum StopBits {
		None, One, OnePointFive, Two

	}

	public enum FlowMode {
		FLOW_OFF, FLOW_XONXOFF, FLOW_HARDWARE
	}

	int timeOut = -1;
	int baudrate;
	int dataBits;
	Ports port;
	Parity parity;
	StopBits stopBit;

	public IOUART(Ports port, int baudrate, Parity parity, int dataBits, StopBits stopBit) {
		this.port = port;
		this.baudrate = baudrate;
		this.parity = parity;
		this.dataBits = dataBits;
		this.stopBit = stopBit;

	}

	@Override
	protected boolean IOOpen() {
		nativeId = NCreate();
		boolean result = NOpen(nativeId, port.toString(), baudrate, parity.ordinal(), dataBits, stopBit.ordinal(), true);
		if (!result) {
			IOLog.log(port + " not opened!");
		}
		return result;
	}

	@Override
	protected boolean IOClose() {
		flush(FlushMode.IN_OUT);
		boolean result = NClose(nativeId);
		NRelease(nativeId);
		if (!result) {
			IOLog.log(port + " not closed!");
		} else {
			nativeId = -1;
		}
		return result;
	}

	@Override
	protected int IOAvailable() {
		return NAvailable(nativeId);
	}

	@Override
	protected int IOFlush(FlushMode mode) {
		return NFlush(nativeId, mode.ordinal());
	}

	@Override
	protected int IORead(int timeOut) {
		return NRead(nativeId);
	}

	@Override
	protected int IORead(byte[] buffer, int offset, int size, int timeOut) {
		long startTime = System.currentTimeMillis();
		int index = 0;
		if (timeOut == -1) {
			return NRead(nativeId, buffer, offset, size);
		} else {
			do {
				if (this.available() > 0) {
					int result = NRead(nativeId, buffer, offset + index, size - index);
					if (result > 0) {
						index += result;
					}
				} else {
					try {
						Thread.sleep(5);
					} catch (InterruptedException e) {

					}
				}
			} while (size - index > 0 && (startTime + timeOut > System.currentTimeMillis() || available() > 0) && this.isOpened());
		}
		return index > 0 ? index : -1;
	}

	@Override
	protected int IOWrite(byte[] buffer, int offset, int size) {
		return NWrite(nativeId, buffer, offset, size);
	}

	public Ports getPort() {
		return port;
	}

	public int getBaudrate() {
		return baudrate;
	}

	public Parity getParity() {
		return parity;
	}

	public int getDataBits() {
		return dataBits;
	}

	public StopBits getStopBit() {
		return stopBit;
	}

	public int getTimeOut() {
		return timeOut;
	}

	public void setTimeOut(int timeOut) {
		this.timeOut = timeOut;
	}

	public int setFlowControl(FlowMode mode) {
		return NSetFlowControl(nativeId, mode.ordinal());
	}

	public boolean setBaudrate(int baudrate) {
		this.baudrate = baudrate;
		boolean ret = NSetBaudrate(nativeId, baudrate);
		try {
			Thread.sleep(10);
		} catch (InterruptedException e) {
			IOLog.logE(e);
		}
		return ret;
	}

	private native long NCreate();

	private native void NRelease(long nativeId);

	private native boolean NOpen(long nativeId, String port, int baudrate, int parity, int dataBits, int stopBit, boolean blocking);

	private native boolean NClose(long nativeId);

	private native int NWrite(long nativeId, byte[] data, int offset, int lenght);

	private native int NRead(long nativeId, byte[] data, int offset, int lenght);

	private native int NRead(long nativeId);

	private native int NFlush(long nativeId, int flush);

	private native int NAvailable(long nativeId);

	private native int NSetReadTimeOut(long nativeId, int timeOut);

	private native boolean NSetBaudrate(long nativeId, int baudrate);

	private native int NSetFlowControl(long nativeId, int xonxoff);

	private native boolean NWaitEvent(long nativeId, long timeOut);

	protected native void NSetGPIO(long nativeId, long gpioOut);
}
